package com.bpcl.lpg.core.strategies.impl;

import org.slf4j.LoggerFactory;
import java.io.InputStream;
import java.io.FileInputStream;
import java.util.Map;
import java.util.HashMap;
import java.util.stream.Stream;
import java.nio.file.Path;
import de.hybris.platform.catalog.model.CatalogUnawareMediaModel;
import java.io.FileNotFoundException;
import java.io.IOException;
import de.hybris.platform.servicelayer.media.NoDataAvailableException;
import org.apache.commons.lang.StringUtils;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.Writer;
import de.hybris.platform.util.CSVWriter;
import java.io.FileWriter;
import java.nio.file.Files;
import java.nio.file.attribute.FileAttribute;
import java.util.Calendar;
import java.text.SimpleDateFormat;
import com.bpcl.lpg.core.model.LPGConnectionExportCronjobModel;
import de.hybris.platform.core.model.media.MediaModel;
import de.hybris.platform.servicelayer.model.ModelService;
import de.hybris.platform.servicelayer.media.MediaService;
import org.slf4j.Logger;
import com.bpcl.lpg.core.strategies.LPGConnectionImpexWriterStrategy;

public class DefaultLPGConnectionImpexWriterStrategy implements LPGConnectionImpexWriterStrategy
{
    private static final Logger LOG;
    private static final String CONSUMER_TYPE_COMMERCIAL = "COMMERCIAL";
    private static final String COMMERCIAL_MEDIA_UID = "lpgCommercialConnection";
    private static final String DOMESTIC_MEDIA_UID = "lpgDomesticConnection";
    private static final String CONSUMER_TYPE_DOMESTIC = "Domestic";
    private static final String IMPEX_FILE_SUFFIX = ".impex";
    private static final String IMPEX_MODE = "INSERT_UPDATE ";
    private static final String IMPEX_LPG_DOMESTIC_CONNECTION_TYPE = "LPGDomesticConnection";
    private static final String IMPEX_LPG_COMMERCIAL_CONNECTION_TYPE = "LPGCommercialConnection";
    private static final String IMPEX_HEADER_1 = "bpclCustomer(uid)";
    private static final String IMPEX_HEADER_2 = "lpgId[unique=true]";
    private static final String IMPEX_HEADER_3 = "consumerID[unique=true]";
    private static final String IMPEX_HEADER_4 = "consumerNumber[unique=true]";
    private static final String IMPEX_HEADER_5 = "consumerStatus(code)";
    private static final String IMPEX_HEADER_6 = "connectionType";
    private static final String IMPEX_HEADER_7 = "distId";
    private static final String IMPEX_HEADER_8 = "lpgpaymentInfos(pk)";
    private static final String IMPEX_HEADER_9 = "codRefills(pk)";
    private String delimiter;
    private Long numberOfLinesToSkip;
    private Integer attributesLength;
    private MediaService mediaService;
    private Integer uidIndex;
    private Integer lpgIdIndex;
    private Integer consumerIDIndex;
    private Integer consumerNumberIndex;
    private Integer consumerStatusIndex;
    private Integer connectionTypeIndex;
    private Integer distIdIndex;
    private Integer lpgpaymentInfosIndex;
    private Integer codRefillsIndex;
    private Integer consumerTypeIndex;
    private ModelService modelService;
    
    public boolean processCustomerCSVAndCreateLPGConnectionImpex(final MediaModel customerCSVMediaModel, final LPGConnectionExportCronjobModel model) {
        final String mediaCodeDomestic = "lpgDomesticConnection" + new SimpleDateFormat("_EEEdMMMyyyyHH:mm:ss").format(Calendar.getInstance().getTime());
        final CatalogUnawareMediaModel lpgDomesticConnectionCSV = this.prepareMedia(mediaCodeDomestic, mediaCodeDomestic + ".impex", "text/x-comma-separated-values");
        final String mediaCodeCommercial = "lpgCommercialConnection" + new SimpleDateFormat("_EEEdMMMyyyyHH:mm:ss").format(Calendar.getInstance().getTime());
        final CatalogUnawareMediaModel lpgCommercialConnectionCSV = this.prepareMedia(mediaCodeCommercial, mediaCodeCommercial + ".impex", "text/x-comma-separated-values");
        String absolutePathDomestic = null;
        FileWriter csvWriterDomestic = null;
        String absolutePathCommercial = null;
        FileWriter csvWriterCommercial = null;
        try {
            final Path pathDomestic = Files.createTempFile("lpgDomesticConnection", ".impex", (FileAttribute<?>[])new FileAttribute[0]);
            absolutePathDomestic = pathDomestic.toAbsolutePath().toString();
            csvWriterDomestic = new FileWriter(pathDomestic.toFile());
            final CSVWriter csvLPGDomesticConnection = new CSVWriter((Writer)csvWriterDomestic);
            final Path pathCommercial = Files.createTempFile("lpgCommercialConnection", ".impex", (FileAttribute<?>[])new FileAttribute[0]);
            absolutePathCommercial = pathCommercial.toAbsolutePath().toString();
            csvWriterCommercial = new FileWriter(pathCommercial.toFile());
            final CSVWriter csvLPGCommercialConnection = new CSVWriter((Writer)csvWriterCommercial);
            final BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(this.getMediaService().getStreamFromMedia(customerCSVMediaModel)));
            final Stream<String> lines = bufferedReader.lines();
            this.writeImpexHeader("LPGDomesticConnection", csvLPGDomesticConnection);
            this.writeImpexHeader("LPGCommercialConnection", csvLPGCommercialConnection);
            lines.skip(this.getNumberOfLinesToSkip()).filter(line -> StringUtils.isNotBlank(line)).forEach(line -> this.processRow(line, csvLPGDomesticConnection, csvLPGCommercialConnection));
            csvLPGDomesticConnection.close();
            csvLPGCommercialConnection.close();
        }
        catch (final NoDataAvailableException | IllegalArgumentException | IOException e) {
            DefaultLPGConnectionImpexWriterStrategy.LOG.error("Could Not prepare impex file", e);
            return Boolean.FALSE;
        }
        finally {
            if (csvWriterDomestic != null) {
                try {
                    csvWriterDomestic.close();
                }
                catch (final IOException ioe) {
                    ioe.printStackTrace();
                }
            }
            if (csvWriterCommercial != null) {
                try {
                    csvWriterCommercial.close();
                }
                catch (final IOException ioe) {
                    ioe.printStackTrace();
                }
            }
        }
        try {
            this.createLPGConnectionCSVMedia(model, absolutePathDomestic, lpgDomesticConnectionCSV, absolutePathCommercial, lpgCommercialConnectionCSV);
        }
        catch (final FileNotFoundException e2) {
            e2.printStackTrace();
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }
    
    protected void processRow(final String line, final CSVWriter csvLPGDomesticConnection, final CSVWriter csvLPGCommercialConnection) {
        final String[] customerAttributes = line.split(this.delimiter);
        final String customerId = StringUtils.trim(customerAttributes[this.getUidIndex()]);
        final String consumerType = StringUtils.trim(customerAttributes[this.getConsumerTypeIndex()]);
        if ("Domestic".equals(consumerType)) {
            this.writeToImpex(customerAttributes, csvLPGDomesticConnection);
        }
        else if ("COMMERCIAL".equals(consumerType)) {
            this.writeToImpex(customerAttributes, csvLPGCommercialConnection);
        }
        else {
            DefaultLPGConnectionImpexWriterStrategy.LOG.error("Invalid Customer {} :", customerId);
            DefaultLPGConnectionImpexWriterStrategy.LOG.error("Invalid Consumer Type {} :", consumerType);
        }
    }
    
    protected void writeToImpex(final String[] customerAttributes, final CSVWriter csv) {
        try {
            int i = 0;
            final Map<Integer, String> line = new HashMap<Integer, String>();
            line.put(i++, "");
            if (customerAttributes.length > this.getUidIndex()) {
                line.put(i++, StringUtils.trim(customerAttributes[this.getUidIndex()]));
            }
            if (customerAttributes.length > this.getLpgIdIndex()) {
                line.put(i++, StringUtils.trim(customerAttributes[this.getLpgIdIndex()]));
            }
            if (customerAttributes.length > this.getConsumerIDIndex()) {
                line.put(i++, StringUtils.trim(customerAttributes[this.getConsumerIDIndex()]));
            }
            if (customerAttributes.length > this.getConsumerNumberIndex()) {
                line.put(i++, StringUtils.trim(customerAttributes[this.getConsumerNumberIndex()]));
            }
            if (customerAttributes.length > this.getConsumerStatusIndex()) {
                line.put(i++, StringUtils.trim(customerAttributes[this.getConsumerStatusIndex()]));
            }
            if (customerAttributes.length > this.getConnectionTypeIndex()) {
                line.put(i++, StringUtils.trim(customerAttributes[this.getConnectionTypeIndex()]));
            }
            if (customerAttributes.length > this.getDistIdIndex()) {
                line.put(i++, StringUtils.trim(customerAttributes[this.getDistIdIndex()]));
            }
            if (customerAttributes.length > this.getLpgpaymentInfosIndex()) {
                line.put(i++, StringUtils.trim(customerAttributes[this.getLpgpaymentInfosIndex()]));
            }
            if (customerAttributes.length > this.getCodRefillsIndex()) {
                line.put(i++, StringUtils.trim(customerAttributes[this.getCodRefillsIndex()]));
            }
            csv.write((Map)line);
        }
        catch (final IOException e) {
            DefaultLPGConnectionImpexWriterStrategy.LOG.error("Exception occurred while writing Impex ");
        }
    }
    
    protected void writeImpexHeader(final String impexHeaderType, final CSVWriter csv) {
        try {
            int i = 0;
            final Map<Integer, String> line = new HashMap<Integer, String>();
            line.put(Integer.valueOf(i++), "INSERT_UPDATE " + impexHeaderType);
            line.put(i++, "bpclCustomer(uid)");
            line.put(i++, "lpgId[unique=true]");
            line.put(i++, "consumerID[unique=true]");
            line.put(i++, "consumerNumber[unique=true]");
            line.put(i++, "consumerStatus(code)");
            line.put(i++, "connectionType");
            line.put(i++, "distId");
            line.put(i++, "lpgpaymentInfos(pk)");
            line.put(i++, "codRefills(pk)");
            csv.write((Map)line);
        }
        catch (final IOException e) {
            DefaultLPGConnectionImpexWriterStrategy.LOG.error("Exception occurred while writing Impex ");
        }
    }
    
    protected void createLPGConnectionCSVMedia(final LPGConnectionExportCronjobModel cronjobModel, final String absolutePathDomestic, final CatalogUnawareMediaModel lpgDomesticConnectionCSV, final String absolutePathCommercial, final CatalogUnawareMediaModel lpgCommercialConnectionCSV) throws FileNotFoundException {
        this.getModelService().save((Object)lpgDomesticConnectionCSV);
        this.getModelService().save((Object)lpgCommercialConnectionCSV);
        this.getMediaService().setStreamForMedia((MediaModel)lpgDomesticConnectionCSV, (InputStream)new FileInputStream(absolutePathDomestic));
        this.getMediaService().setStreamForMedia((MediaModel)lpgCommercialConnectionCSV, (InputStream)new FileInputStream(absolutePathCommercial));
        this.getModelService().save((Object)lpgDomesticConnectionCSV);
        this.getModelService().save((Object)lpgCommercialConnectionCSV);
        cronjobModel.setLpgDomesticConnectionMedia(lpgDomesticConnectionCSV);
        cronjobModel.setLpgCommercialConnectionMedia(lpgCommercialConnectionCSV);
        this.getModelService().save((Object)cronjobModel);
    }
    
    private CatalogUnawareMediaModel prepareMedia(final String code, final String reportFileName, final String mimeType) {
        final CatalogUnawareMediaModel media = (CatalogUnawareMediaModel)this.modelService.create((Class)CatalogUnawareMediaModel.class);
        media.setCode(code);
        media.setMime(mimeType);
        media.setRealFileName(reportFileName);
        return media;
    }
    
    public String getDelimiter() {
        return this.delimiter;
    }
    
    public void setDelimiter(final String delimiter) {
        this.delimiter = delimiter;
    }
    
    public Long getNumberOfLinesToSkip() {
        return this.numberOfLinesToSkip;
    }
    
    public void setNumberOfLinesToSkip(final Long numberOfLinesToSkip) {
        this.numberOfLinesToSkip = numberOfLinesToSkip;
    }
    
    public Integer getAttributesLength() {
        return this.attributesLength;
    }
    
    public void setAttributesLength(final Integer attributesLength) {
        this.attributesLength = attributesLength;
    }
    
    public MediaService getMediaService() {
        return this.mediaService;
    }
    
    public void setMediaService(final MediaService mediaService) {
        this.mediaService = mediaService;
    }
    
    public Integer getUidIndex() {
        return this.uidIndex;
    }
    
    public void setUidIndex(final Integer uidIndex) {
        this.uidIndex = uidIndex;
    }
    
    public Integer getLpgIdIndex() {
        return this.lpgIdIndex;
    }
    
    public void setLpgIdIndex(final Integer lpgIdIndex) {
        this.lpgIdIndex = lpgIdIndex;
    }
    
    public Integer getConsumerIDIndex() {
        return this.consumerIDIndex;
    }
    
    public void setConsumerIDIndex(final Integer consumerIDIndex) {
        this.consumerIDIndex = consumerIDIndex;
    }
    
    public Integer getConsumerNumberIndex() {
        return this.consumerNumberIndex;
    }
    
    public void setConsumerNumberIndex(final Integer consumerNumberIndex) {
        this.consumerNumberIndex = consumerNumberIndex;
    }
    
    public Integer getConsumerStatusIndex() {
        return this.consumerStatusIndex;
    }
    
    public void setConsumerStatusIndex(final Integer consumerStatusIndex) {
        this.consumerStatusIndex = consumerStatusIndex;
    }
    
    public Integer getConnectionTypeIndex() {
        return this.connectionTypeIndex;
    }
    
    public void setConnectionTypeIndex(final Integer connectionTypeIndex) {
        this.connectionTypeIndex = connectionTypeIndex;
    }
    
    public Integer getDistIdIndex() {
        return this.distIdIndex;
    }
    
    public void setDistIdIndex(final Integer distIdIndex) {
        this.distIdIndex = distIdIndex;
    }
    
    public Integer getLpgpaymentInfosIndex() {
        return this.lpgpaymentInfosIndex;
    }
    
    public void setLpgpaymentInfosIndex(final Integer lpgpaymentInfosIndex) {
        this.lpgpaymentInfosIndex = lpgpaymentInfosIndex;
    }
    
    public Integer getCodRefillsIndex() {
        return this.codRefillsIndex;
    }
    
    public void setCodRefillsIndex(final Integer codRefillsIndex) {
        this.codRefillsIndex = codRefillsIndex;
    }
    
    public Integer getConsumerTypeIndex() {
        return this.consumerTypeIndex;
    }
    
    public void setConsumerTypeIndex(final Integer consumerTypeIndex) {
        this.consumerTypeIndex = consumerTypeIndex;
    }
    
    public ModelService getModelService() {
        return this.modelService;
    }
    
    public void setModelService(final ModelService modelService) {
        this.modelService = modelService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(DefaultLPGConnectionImpexWriterStrategy.class);
    }
}
